#####################################################################
# 
# TQInterface.py - Zope Transactional Queue Interfaces
#
# $Id: TQInterface.py,v 1.5 2003/11/26 17:07:44 kevin Exp $
#
# Copyright (C) 2003 Kevin Gill
#
# Derived from the Xron version 0.0.10
# Copyright (C) 2000 Loren Stafford
#  
# Xron is Derived from the ZScheduler product version 0.0.7 in
# accord with the terms of the ZScheduler's license.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the 
#     Free Software Foundation, Inc., 
#     59 Temple Place - Suite 330, 
#     Boston, MA  02111-1307, USA.
# 
#####################################################################

""" Zope Transactional Queue - Interface definitions

This file contains the interface definitions for the components of
the Transactional Queue product.
"""


from Interface import Base

class IPollingThread(Base):
    def registerObject(self, obj):
        """Adds a new object to the system"""
    def unregisterObject(self, obj):
        """Drops a object from the system"""
    def reregisterObject(self, old_path, obj):
        """Called when a object has been moved"""
    def items(self):
        """Returns the list of object paths and their
        status - for ZMI. """
    def trigger(self):
        """If the thread is idling (in 10 second wait),
        the kick it off."""

class IPolledObject(Base):
    def Process(self, app, private_data):
        """This method is called by IPollingThread at regular
        intervals to do work. private_data contains a dictionary
        which is stored in the polling_thread so if you have
        data which needs to be preserved over calls, e.g.
        retry counters, save it here. 
        
        The return value is an indicator of work remaining. If
        the return value is 0, the PollingThread will sleep for
        10 seconds (configurable) before calling this method
        again. If the counter is non zero, it will sleep for
        .1 seconds (configurable) before calling this method again."""
    def getStatusMessage(self):
        """Returns a string which is displayed in the IPollingThread
        ZMI for the status of this object."""

class IZTxnQueue(Base):
    def enqueue(self, Data):
        """Enqueues a piece of data. Returns an identifier."""
    def dequeue(self):
        """Pops an item off the queue. """
    def items(self):
        """Returns the queue - for ZMI. """
    def sleep(self, seconds):
        """Sleep for a number of seconds - used for testing. """
    def find(self, OID):
        """Returns a specific object or None"""
    def block(self):
        """Block processing"""
    def unblock(self):
        """Unblock processing"""

    # I defined these to make subclassing easier
    # They are not part of the API
    def HookStartProcessing(self, App, private_data, Count):
        """Called a the start of processing, 1 to continue, 0
        to exit and wait for the next time round"""
    def HookEndProcessing(self, App, private_data, Count):
        """This is called at the end of processing. It sets
        the return value. 0 - I have no more work to do,
        1 - I am busy, call me back as soon as possible"""
    def HookProcessOne(self, Data, OID, private_data):
        """This method is called to process a single queue
        item. It is passed the queued data, the queued object
        id and the ProcessingObject returned by
        HookGetProcessingFunction."""
    def HookException(self, App, private_data):
        """This method is called to if an exception occurs,
        which causes the queue to be blocked. It is intended for
        use in notifying an administrator."""


